/**
 * Abstract Enemy class. 
 * Moves, shoots Bullets and takes damage.
 */
import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

public abstract class Enemy extends Actor
{
    int speed;               //How fast the enemy travels
    int points;              //How many points a kill is worth
    int shootTimer;          //countdown / timer = how many times Enemy attacks
    int health;              //How much damage needs to kill it 
    boolean isBoss;          //Best boolean ever.
    int shootCountdown = 0;  //How many ticks before the Enemy attacks
    
    //random border and boolean for moving enemies
    int border ;
    boolean truth;
    
    //Costumes
    GreenfootImage[] costumes; 
    int costumeNumber;
    
    public void act()
    {
        //changes costumes
        costumeNumber++;
        if(costumeNumber / 3 > 3)
            costumeNumber = 0;
            
        //if it flew off the bottom of the screen,
        if (getY() >= getWorld().getHeight())       
        {   
            getWorld().removeObject(this);      //delete it
        } 
        else 
        {    
            move(speed);     
            //This times the Enemy's shooting
            if (shootCountdown % shootTimer == 0)   
                attack();                     
            
            shootCountdown++;
            checkDamaged(); //check for collisions with MyBullets
        }
        setImage(costumes[costumeNumber / 3]);
    }
    
    //Checks whether Enemy collides with a MyProjectile, lethal Star, Shield or Player.
    public void checkDamaged()
    {
        MyProjectile m = (MyProjectile) getOneIntersectingObject(MyProjectile.class);
        Star s         = (Star)   getOneIntersectingObject(Star.class);
        Player pl      = (Player) getOneIntersectingObject(Player.class);
        Shield sh      = (Shield) getOneIntersectingObject(Shield.class);
        
        Player p               = getPlayer();
        EnvironmentGenerator g = getEnvironmentGenerator();
        
        Explosion splode = new Explosion("small");
        
        //try to find an intersecting Projectile or Shield object
        
        if (m!=null) //if we found a Projectile intersecting us ...
        {             
            getWorld().addObject(splode, m.getX(), m.getY()); //create an explosion...
            getWorld().removeObject(m);                      //remove the Projectile...
            
            if(isBoss)
            {
                p.addScore(1);                              //1 point per hit on Boss
            }
            
            if(health > 0) //if Enemy still has health...
            {
                loseHealth(m.getDamage());                    //lose health according 
                                                             //to bullet's power
                costumeNumber = 12;                           //show damaged costume
            }
        }
        
        if (s != null && s.getLethal()) //if a BlackHole has been activated and a Star collides....
        {
            getWorld().addObject(splode, s.getX(), s.getY()); //create an explosion...
            getWorld().removeObject(s);
            
            if(health > 0) //if Enemy still has health...
            {
                loseHealth(1);                                 //lose 1 health
                costumeNumber = 12;                           //show damaged costume
            }
        }
        
        //Instakill if hit by Player or Shield
        if (sh!= null || pl!= null)
        {
            getWorld().addObject(splode, getX(), getY()); 
            loseHealth(20);                                 //loss of 20 life ensures Enemy death
                                                            //Only injures BossEnemy
        }
            
        checkHealth();
    }
    
    //removes itself from the world if health is 0.
    public void checkHealth()
    {
        if(health <= 0)
        {
            Player p = getPlayer();
            p.addScore(points);   
            getWorld().removeObject(this);  
        }
    }

    public abstract void attack();
   
    //custom move method
    public void move(int y)
    {
        setLocation(getX(), getY() + y);
    }
    
    public void loseHealth(int x)
    {
        health -= x;
    }
    
    public Player getPlayer()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getPlayer();
    } 
    
    public EnvironmentGenerator getEnvironmentGenerator()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getEnvironmentGenerator();
    } 
    
    public int getHealth()
    {
        return health;
    }
}
    
/*PUSH ME
 *AND THEN JUST TOUCH ME
 *TIL I CAN GET MY 
 *SATISFACTION
 */