import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * Draws Stars towards itself causing destruction.
 * (will not collide with Enemys)
 */
public class MyBlackHole extends Actor
{    
    private int timer = 0;
    private int countdown = 140;        //Acts until BlackHole runs out
    private int change = 5;             //Rate of costume change
    private boolean activated = false; 
    private boolean safety = false;     //Ensures BlackHole won't go off twice.
    int speed;                          //Speed of bullet
                        
    GreenfootImage[] costumes = new GreenfootImage[3];
    int costumeNumber = 0; 
    
    public MyBlackHole() 
    {
        speed = 3;
        setRotation(-90);
        
        for (int i = 0; i < 3; i++) 
        {
            //SET TO BOMB COSTUME 
            costumes[i] = new GreenfootImage("BombBlue_0"+(i+1)+".gif");
        }
        //System.out.println(damage);
    }
    
    public void act()
    {        
        EnvironmentGenerator g = getEnvironmentGenerator();
        
        if(timer >= 80)
        {
            change = 1;
        }
        
        if(timer % change == 0)
        {
            setImage(costumes[costumeNumber]);
            costumeNumber++;
            if(costumeNumber > 2){
                 costumeNumber = 0;
            }
        }
        
        //Stops moving when activated.
        if(!activated){
            move(speed);
        }
        
        //Only goes off after a certain time.
        //Stops Star production temporarily.
        if(timer >= 100)  
        {
            if(!safety){
                activated = true;   
                goZoom();
                g.pause();
                
            }
            safety = true;
        }  

        if(activated)
        {
            countdown--;
        }
        
        //Restarts Star production and destroys itself.
        if(offScreen() || (activated && countdown <= 0))
        {
            g.go();
            activated = false;
            Explosion ex = new Explosion("big");
            getWorld().addObject(ex, getX(), getY());
            goBoom();
            getWorld().removeObject(this);
        }
        
        timer++;
    }
    
    public EnvironmentGenerator getEnvironmentGenerator()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getEnvironmentGenerator();
    }
    
    public boolean offScreen()
    //checks to see if the bullet is off screen or not
    {
        int width = getWorld().getWidth(); //width of screen
        int height = getWorld().getHeight(); //height of screen
        int x = getX(); //bullet's x
        int y = getY(); //bullet's y
        return x > width || x < 0 || y > height || y < 0;
    }
    
    public void goBoom()
    {
        GreenfootSound laser = new GreenfootSound("cowmanglermain.wav");
        laser.setVolume(80);
        laser.play();
    }
    
    public void goZoom()
    {
        GreenfootSound zoom = new GreenfootSound("teleporter_send.wav");
        zoom.setVolume(100);
        zoom.play();
    }
}