import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

public class Player extends Actor 
{
    int speed;  //This keeps track of how fast we can move
    Gun gun; //This is our gun!
    Launcher launcher;
    boolean intro, invul, cheats;
    private int attack;     //Upgradable attack strength of the ship.
    private int score;
    private int multi;      //Score multiplier
    private int pow, gunType, missleCount;
    
    GreenfootImage[] costumes = new GreenfootImage[11]; //Costumes
    int costumeNumber; 
    
    //Constructor!  Initializes the default Player object
    public Player()
    {
        //setting the costumes
        for (int i = 0; i < 11; i++)
        {
            costumes[i] = new GreenfootImage("jets_"+(i+1)+".gif");
        }
        
        costumeNumber = 5; //default costume
        intro       = true;
        
        gun         = new BasicGun(this);
        launcher    = new MissleLauncher(this);
        
        //stats
        speed       = 5;
        invul       = false;
        
        attack      = 1;
       
        //temp powerups
        pow         = 0;
        gunType     = 0;
        missleCount = 0;
        
        score       = 0;
        multi       = 1;
        
        cheats      = false;
    }    
     
    
    public int getSpeed()
    {
        return speed;
    }
    public int getAttack()
    {
        return attack + 1;  //returns attack plus 1 or else it glitches....
                           
    }
    public int getScore()
    {
        return score;
    }
    
    public int getGunType()
    {
        return gunType;
    }
    
    public LifeCounter getLifeCounter()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getLifeCounter();
    }
    public Scoreboard getScoreboard()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getScoreboard();
    }
    

    //Act - do whatever the Player wants to do. This method is called whenever
    //the 'Act' or 'Run' button gets pressed in the environment.
    public void act() 
    {
        checkKeys();
        checkOutOfBounds();
        setImage(costumes[costumeNumber]);
        
        gun.updateCooldown();
        launcher.updateCooldown();
        
        if(!intro) //Don't do these during the introscreen
        {
            checkDamaged(); //check whether hit by bullet 
            checkPickup();  //check whether hit by powerup
            checkDead();    //check whether life = 0
        }
        
        invul = false;
    }    
    
    //handles keyboard input
    public void checkKeys()
    {
        if( !Greenfoot.isKeyDown("left") || !Greenfoot.isKeyDown("right") )
        {
            returnCostume();
        }
        if(Greenfoot.isKeyDown("up"))
        {
            move(0,-speed);  //moves in x, y direction
        }
        if(Greenfoot.isKeyDown("down"))
        {
            move(0, speed);  
        }
        if(Greenfoot.isKeyDown("left"))
        {
            move(-speed,0);
        }
        if(Greenfoot.isKeyDown("right"))
        {
            move(speed, 0);
        }
        if(Greenfoot.isKeyDown("space"))
        {
            gun.shoot();
            launcher.shoot(missleCount);
        }
        
        //change costume
        if(Greenfoot.isKeyDown("left"))
        {
            costumeNumber--;
        }
        if(Greenfoot.isKeyDown("right"))
        {
            costumeNumber++;
        }
        
        //change weapons. TESTING ONLY.
        
        if(cheats)
        {
            if(Greenfoot.isKeyDown("1"))
            {
                gun = new BasicGun(this);
                gunType = 0;
            }
            if(Greenfoot.isKeyDown("2"))
            {
                gun = new MachineGun(this);
                gunType = 1;
            }
            if(Greenfoot.isKeyDown("3"))
            {
                gun = new ShotGun(this);
                gunType = 2;
            }
            if(Greenfoot.isKeyDown("4"))
            {
                gun = new ScatterGun(this);
                gunType = 3;
            }
            if(Greenfoot.isKeyDown("5"))
            {
                gun = new BlackHoleGun(this);
                gunType = 4;
            }
            if(Greenfoot.isKeyDown("6"))
            {
                gun = new NovaGun(this);
                gunType = 5;
            }
            
            if(Greenfoot.isKeyDown("8"))
            {
                addMissle();
            }
            if(Greenfoot.isKeyDown("9"))
            {
                LifeCounter lc = getLifeCounter();
                lc.addLife();
            }
            if(Greenfoot.isKeyDown("0"))
            {
                LifeCounter lc = getLifeCounter();
                lc.setLives(0);
            }
        }
    }
    
    //custom move method for moving up right left down
    public void move(int x, int y)
    {
        setLocation(getX() + x, getY() + y);
    }
    
    public void checkOutOfBounds()
    {
        int width = getWorld().getWidth();
        int height = getWorld().getHeight();
        int x = getX();
        int y = getY();
        
        if(y > height - 11)
            setLocation(x, height - 11);
        if(y < 0)
            setLocation(x, 0);
        if(x > width)
            setLocation(width, y);
        if(x < 0) 
            setLocation(0, y);
    }
    
    
    //checks whether ship is hit by enemy bullet or an Enemy (similar to the Enemy's method)
    public void checkDamaged()
    {
        EnemyProjectile b = (EnemyProjectile) getOneIntersectingObject(EnemyProjectile.class);
        Enemy e  = (Enemy)  getOneIntersectingObject(Enemy.class);
        LifeCounter lc   = getLifeCounter();
        Explosion splode = new Explosion("small");
        
        // try to find an intersecting Bullet or Enemy object
        if ( (e != null || b!= null) && invul == false) //if an Enemy or a Bullet hits...
        {
            invul = true;
            //if(e != null) getWorld().removeObject(e);
            if(b != null) getWorld().removeObject(b);
            
            lc.loseLife(); //lose a life...
            getWorld().addObject(splode, getX(), getY()); //create an explosion...
            getImage().setTransparency(0);  //and hide the player...
            //work on this
            getImage().setTransparency(255);
        }
    }
    
    public void addSpeed()
    {
        speed++;
    }
    
    public void addAttack()
    {
        attack++;
    }
    
    public void addScore(int s)
    {
        score += (s * multi);
    }
    
    public void addMissle()
    {
        if(missleCount < 8)
            missleCount++;
    }
    
    public void endIntro()
    {
        intro = false;
    }
    
    //checks whther ship grabs a powerup 
    public void checkPickup()
    {  
        Powerup w =(Powerup) getOneIntersectingObject(Powerup.class);
        
        if(w != null)
        {
            pow = w.getType();
            //not int pow LOL
            if(w!=null) getWorld().removeObject(w);
            
            addScore(50);
            
            setPower();
            setGun();
        }
    }
    
    
    //Sets the Player's power or gun depending on the Powerup type.
    public void setPower()
    {
 
        if     (pow == 0){
            //BACK TO NORMAL
            setDefault();
            System.out.println("RESET!");
        }
        //SUPPORT
        else if(pow == 1){
            //FULL HEALTH
            LifeCounter lc = getLifeCounter();
            lc.setLives(lc.getMaxLives());
        }
        else if(pow == 2){
            //EXTRA LIFE 
            LifeCounter lc = getLifeCounter();
            lc.addLife();
        }
        else if(pow == 3){
            //MAKE A SHIELD
            Shield s = new Shield();
            getWorld().addObject(s, this.getX(), this.getY());
        }
        else if(pow == 4){
            //ADDITIONAL MISSLES
            addMissle();
        }
        //GUNS
        else if(pow == 5){
            //SET TO MACHINEGUN
            gunType = 1;
        }
        else if(pow == 6){
            //SET TO SHOTGUN
            gunType = 2;
        }
        else if(pow == 7){
            //SET TO SCATTERGUN
            gunType = 3;
        }
        else if(pow == 8){
            //SET TO BLACKHOLEGUN
            gunType = 4;
        }
        else{
            //SET TO NOVAGUN (if something goes wrong, i guess)
            gunType = 99;
            System.out.println("ERROR!");
        }
        
    }
    
    public void setGun()
    {
        if     (gunType == 0)
            gun = new BasicGun(this);
        else if(gunType == 1)
            gun = new MachineGun(this);
        else if(gunType == 2)
            gun = new ShotGun(this);
        else if(gunType == 3)
            gun = new ScatterGun(this);
        else if(gunType == 4)
            gun = new BlackHoleGun(this);
        else
            gun = new NovaGun(this);
    }
    
    public void setDefault()
    {
        gun = new BasicGun(this);
        
        pow     = 0;
        gunType = 0;
        multi   = 1;
    }
        
    
    public void checkDead()
    {
        LifeCounter lc = getLifeCounter();
        
        if(lc.getLives() <= 0)
        {
            //If lives = 0, display game over
            Greenfoot.playSound("rocket_blackbox_explode3.wav");
            
            Kaboom ka = new Kaboom();
            getWorld().addObject(ka, getX(), getY());
            
            GameOver ga = new GameOver();
            getWorld().addObject(ga, 250, 250);
            
            getWorld().removeObject(this);
        }
    }
    
    //returns the costume back to default
    public void returnCostume()
    {
        if(costumeNumber < 5) costumeNumber++;
        if(costumeNumber > 5) costumeNumber--;
    }
    
    public void setCheats()
    {
        cheats = true;
    }
}
