/**
 * Cosmetic Star (SPACE DUST) effect.
 * Turns lethal from BlackHole.
 */
import greenfoot.*;
import java.util.List;

public class Star extends Actor
{
    private int speed;
    private int direction;
    private boolean lethal = false;
   
    public Star()
    {
        //speed is between 1 and 4
        speed = 1 + Greenfoot.getRandomNumber(3);
        setRotation(90);
    }
    
    public void act()
    {
        EnvironmentGenerator g = getEnvironmentGenerator();  
        
        if(g.getPaused())
        {lethal = true;}
        else
        {lethal = false;}
        
        if(!lethal)
        {
            //regular actions
            if (getY() >= getWorld().getHeight()|| 
                getY() <= -11 || 
                getX() >= getWorld().getWidth() || 
                getX() <= 0) 
            {   
                getWorld().removeObject(this);  
                //if it flew off the screen, delete it
            } 
            else
            {move(speed);}
        }else //when activated
        {       
            speed = 5;
            MyBlackHole h = getBlackHole();
            
            int xb = h.getX();
            int yb = h.getY();
            
            if(h!=null)
            {
                if(xb - getX() == 0)
                    setLocation(xb,yb);
                int degree = Toolkit.absoluteBearing(getX(), getY(), xb, yb);
                setRotation(degree);
               
            }
            
            move(speed);
            checkHit();
        }
    }
    
    //removes itself when touching the BlackHole
    public void checkHit()
    {  
        MyBlackHole h =(MyBlackHole) getOneIntersectingObject(MyBlackHole.class);
        
        if(h != null)
        {
            getWorld().removeObject(this);
        }
    }
    
    public void setSpeed(int s)
    {
        speed = s;
    }
    
    public boolean getLethal()
    {
        return lethal;
    }
    
    public EnvironmentGenerator getEnvironmentGenerator()
    {
        FirstLevel level = (FirstLevel)getWorld();
        return level.getEnvironmentGenerator();
    } 
    
    public MyBlackHole getBlackHole()
    {
        FirstLevel level = (FirstLevel)getWorld();
        List<MyBlackHole> b = level.getObjects(MyBlackHole.class);
        
        //System.out.println("Length: " + b.size());
        
        if(b.size() > 0)
            return b.get(b.size() - 1);
        else
            return null;
    }
}